@@ -1,8 +1,8 @@
 /*
  *  SimInf, a framework for stochastic disease spread simulations
  *  Copyright (C) 2015  Pavol Bauer
- *  Copyright (C) 2015  Stefan Engblom
- *  Copyright (C) 2015  Stefan Widgren
+ *  Copyright (C) 2015 - 2016  Stefan Engblom
+ *  Copyright (C) 2015 - 2016  Stefan Widgren
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "SISe3.h"
+#include "siminf.h"
 #include "siminf_forward_euler_linear_decay.h"
 
 /* Offset in integer compartment state vector */
@@ -169,18 +169,23 @@ double SISe3_I_3_to_S_3(
 /**
  * Update environmental infectious pressure phi
  *
- * @param u The compartment state vector in node.
- * @param v The continuous state vector in node.
+ * @param v_new The continuous state vector in the node after the post
+ * time step
+ * @param u The compartment state vector in the node.
+ * @param v The current continuous state vector in the node.
  * @param ldata The local data vector for the node.
  * @param gdata The global data vector.
  * @param node The node.
  * @param t Current time.
- * @param sd The sub-domain of node.
- * @return 1 if needs update, else 0.
+ * @param sd The sub-domain of the node.
+ * @return error code (<0), or 1 if node needs to update the
+ * transition rates, or 0 when it doesn't need to update the
+ * transition rates.
  */
 int SISe3_post_time_step(
+    double *v_new,
     const int *u,
-    double *v,
+    const double *v,
     const double *ldata,
     const double *gdata,
     int node,
@@ -194,52 +199,34 @@ int SISe3_post_time_step(
 
     /* Time dependent beta in each of the four intervals of the
      * year. Forward Euler step. */
-    v[PHI] = siminf_forward_euler_linear_decay(
+    v_new[PHI] = siminf_forward_euler_linear_decay(
         phi, day,
         ldata[END_T1], ldata[END_T2], ldata[END_T3], ldata[END_T4],
         gdata[BETA_T1], gdata[BETA_T2], gdata[BETA_T3], gdata[BETA_T4]);
 
     if (n > 0.0)
-        v[PHI] += gdata[ALPHA] * I_n / n + gdata[EPSILON];
+        v_new[PHI] += gdata[ALPHA] * I_n / n + gdata[EPSILON];
     else
-        v[PHI] += gdata[EPSILON];
+        v_new[PHI] += gdata[EPSILON];
 
-    /* 1 if needs update */
-    return phi != v[PHI];
+    if (isfinite(v_new[PHI]))
+        return phi != v_new[PHI]; /* 1 if needs update */
+    return SIMINF_ERR_V_IS_NOT_FINITE;
 }
 
 /**
- * Run simulation for the SISe3 model
+ * Run simulation with the SISe3 model
  *
- * This function is called from R with '.Call'
- * @param model The SISe3 model
- * @param threads Number of threads
+ * @param model The SISe3 model.
+ * @param threads Number of threads.
  * @param seed Random number seed.
- * @return S4 class SISe3 with the simulated trajectory in U
+ * @return The simulated trajectory.
  */
 SEXP SISe3_run(SEXP model, SEXP threads, SEXP seed)
 {
-    int err = 0;
-    SEXP result, class_name;
-    PropensityFun t_fun[] = {&SISe3_S_1_to_I_1, &SISe3_I_1_to_S_1,
-                             &SISe3_S_2_to_I_2, &SISe3_I_2_to_S_2,
-                             &SISe3_S_3_to_I_3, &SISe3_I_3_to_S_3};
+    TRFun tr_fun[] = {&SISe3_S_1_to_I_1, &SISe3_I_1_to_S_1,
+                      &SISe3_S_2_to_I_2, &SISe3_I_2_to_S_2,
+                      &SISe3_S_3_to_I_3, &SISe3_I_3_to_S_3};
 
-    if (R_NilValue == model || S4SXP != TYPEOF(model))
-        Rf_error("Invalid SISe3 model");
-
-    class_name = getAttrib(model, R_ClassSymbol);
-    if (strcmp(CHAR(STRING_ELT(class_name, 0)), "SISe3") != 0)
-        Rf_error("Invalid SISe3 model: %s", CHAR(STRING_ELT(class_name, 0)));
-
-    result = PROTECT(duplicate(model));
-
-    err = siminf_run(result, threads, seed, t_fun, &SISe3_post_time_step);
-
-    UNPROTECT(1);
-
-    if (err)
-        siminf_error(err);
-
-    return result;
+    return siminf_run(model, threads, seed, tr_fun, &SISe3_post_time_step);
 }
