@@ -1,7 +1,7 @@
 ## SimInf, a framework for stochastic disease spread simulations
 ## Copyright (C) 2015  Pavol Bauer
-## Copyright (C) 2015  Stefan Engblom
-## Copyright (C) 2015  Stefan Widgren
+## Copyright (C) 2015 - 2016  Stefan Engblom
+## Copyright (C) 2015 - 2016  Stefan Widgren
 ##
 ## This program is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -21,28 +21,108 @@ library(SimInf)
 ## For debugging
 sessionInfo()
 
-init <- structure(list(id = c(0, 1, 2, 3, 4, 5),
-                       S  = c(0, 1, 2, 3, 4, 5),
-                       I  = c(0, 0, 0, 0, 0, 0)),
-                  .Names = c("id", "S", "I"),
-                  row.names = c(NA, -6L), class = "data.frame")
-
-## Check missing columns in init
-res <- tools::assertError(SISe(init = init[, c("S", "I")]))
-stopifnot(length(grep("Missing columns in init",
-                      res[[1]]$message)) > 0)
-res <- tools::assertError(SISe(init = init[, c("id", "I")]))
-stopifnot(length(grep("Missing columns in init",
-                      res[[1]]$message)) > 0)
-res <- tools::assertError(SISe(init = init[, c("id", "S")]))
-stopifnot(length(grep("Missing columns in init",
+## Check invalid u0
+res <- tools::assertError(SISe(u0 = "u0"))
+stopifnot(length(grep("'u0' must be a data.frame",
                       res[[1]]$message)) > 0)
 
-## Check missing upsilon
-res <- tools::assertError(SISe(init    = init,
+u0 <- structure(list(S  = c(0, 1, 2, 3, 4, 5),
+                     I  = c(0, 0, 0, 0, 0, 0)),
+                .Names = c("S", "I"),
+                row.names = c(NA, -6L), class = "data.frame")
+
+## Check missing columns in u0
+res <- tools::assertError(SISe(u0 = u0[, "I", drop = FALSE]))
+stopifnot(length(grep("Missing columns in u0",
+                      res[[1]]$message)) > 0)
+res <- tools::assertError(SISe(u0 = u0[, "S", drop = FALSE]))
+stopifnot(length(grep("Missing columns in u0",
+                      res[[1]]$message)) > 0)
+
+## Check phi
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep("a", nrow(u0)),
+                               upsilon = 0.0357,
+                               gamma   = 0.1,
+                               alpha   = 1.0,
+                               beta_t1 = 0.19,
+                               beta_t2 = 0.085,
+                               beta_t3 = 0.075,
+                               beta_t4 = 0.185,
+                               end_t1  = 91,
+                               end_t2  = 182,
+                               end_t3  = 273,
+                               end_t4  = 365,
+                               epsilon = 0.000011))
+stopifnot(length(grep("Invalid 'phi': must be numeric vector",
+                      res[[1]]$message)) > 0)
+
+res <- tools::assertError(SISe(u0      = u0,
+                               tspan   = seq_len(10) - 1,
+                               events  = NULL,
+                               phi     = matrix(rep(1, nrow(u0))),
+                               upsilon = 0.0357,
+                               gamma   = 0.1,
+                               alpha   = 1.0,
+                               beta_t1 = 0.19,
+                               beta_t2 = 0.085,
+                               beta_t3 = 0.075,
+                               beta_t4 = 0.185,
+                               end_t1  = 91,
+                               end_t2  = 182,
+                               end_t3  = 273,
+                               end_t4  = 365,
+                               epsilon = 0.000011))
+stopifnot(length(grep("Invalid 'phi': must be numeric vector",
+                      res[[1]]$message)) > 0)
+
+res <- tools::assertError(SISe(u0      = u0,
+                               tspan   = seq_len(10) - 1,
+                               events  = NULL,
+                               phi     = rep(1, nrow(u0) - 1),
+                               upsilon = 0.0357,
+                               gamma   = 0.1,
+                               alpha   = 1.0,
+                               beta_t1 = 0.19,
+                               beta_t2 = 0.085,
+                               beta_t3 = 0.075,
+                               beta_t4 = 0.185,
+                               end_t1  = 91,
+                               end_t2  = 182,
+                               end_t3  = 273,
+                               end_t4  = 365,
+                               epsilon = 0.000011))
+stopifnot(length(
+    grep("Invalid 'phi': must be numeric vector with length 'nrow[(]u0[)]'",
+                      res[[1]]$message)) > 0)
+
+res <- tools::assertError(SISe(u0      = u0,
+                               tspan   = seq_len(10) - 1,
+                               events  = NULL,
+                               phi     = rep(-1, nrow(u0)),
+                               upsilon = 0.0357,
+                               gamma   = 0.1,
+                               alpha   = 1.0,
+                               beta_t1 = 0.19,
+                               beta_t2 = 0.085,
+                               beta_t3 = 0.075,
+                               beta_t4 = 0.185,
+                               end_t1  = 91,
+                               end_t2  = 182,
+                               end_t3  = 273,
+                               end_t4  = 365,
+                               epsilon = 0.000011))
+stopifnot(length(
+    grep("Invalid 'phi': must be numeric vector with non-negative values",
+         res[[1]]$message)) > 0)
+
+## Check missing upsilon
+res <- tools::assertError(SISe(u0      = u0,
+                               tspan   = seq_len(10) - 1,
+                               events  = NULL,
+                               phi     = rep(1, nrow(u0)),
                                gamma   = 0.1,
                                alpha   = 1.0,
                                beta_t1 = 0.19,
@@ -58,10 +138,10 @@ stopifnot(length(grep("'upsilon' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing gamma
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                alpha   = 1.0,
                                beta_t1 = 0.19,
@@ -77,10 +157,10 @@ stopifnot(length(grep("'gamma' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing alpha
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                beta_t1 = 0.19,
@@ -96,10 +176,10 @@ stopifnot(length(grep("'alpha' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing beta_t1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -115,10 +195,10 @@ stopifnot(length(grep("'beta_t1' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing beta_t2
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -134,10 +214,10 @@ stopifnot(length(grep("'beta_t2' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing beta_t3
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -153,10 +233,10 @@ stopifnot(length(grep("'beta_t3' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing beta_t4
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -172,10 +252,10 @@ stopifnot(length(grep("'beta_t4' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing end_t1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -191,10 +271,10 @@ stopifnot(length(grep("'end_t1' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing end_t2
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -210,10 +290,10 @@ stopifnot(length(grep("'end_t2' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing end_t3
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -229,10 +309,10 @@ stopifnot(length(grep("'end_t3' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing end_t4
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -248,10 +328,10 @@ stopifnot(length(grep("'end_t4' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing epsilon
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -267,10 +347,10 @@ stopifnot(length(grep("'epsilon' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric upsilon
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = "0.0357",
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -287,10 +367,10 @@ stopifnot(length(grep("'upsilon' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric gamma
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = "0.1",
                                alpha   = 1.0,
@@ -307,10 +387,10 @@ stopifnot(length(grep("'gamma' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric alpha
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = "1.0",
@@ -327,10 +407,10 @@ stopifnot(length(grep("'alpha' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric beta_t1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -347,10 +427,10 @@ stopifnot(length(grep("'beta_t1' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric beta_t2
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -367,10 +447,10 @@ stopifnot(length(grep("'beta_t2' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric beta_t3
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -387,10 +467,10 @@ stopifnot(length(grep("'beta_t3' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric beta_t4
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -407,10 +487,10 @@ stopifnot(length(grep("'beta_t4' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-integer end_t1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -426,10 +506,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t1' must be integer",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -446,10 +526,10 @@ stopifnot(length(grep("'end_t1' must be integer",
                       res[[1]]$message)) > 0)
 
 ## Check non-integer end_t2
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -465,10 +545,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t2' must be integer",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -485,10 +565,10 @@ stopifnot(length(grep("'end_t2' must be integer",
                       res[[1]]$message)) > 0)
 
 ## Check non-integer end_t3
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -504,10 +584,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t3' must be integer",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -524,10 +604,10 @@ stopifnot(length(grep("'end_t3' must be integer",
                       res[[1]]$message)) > 0)
 
 ## Check non-integer end_t4
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -543,10 +623,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t4' must be integer",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -563,10 +643,10 @@ stopifnot(length(grep("'end_t4' must be integer",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric epsilon
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -583,10 +663,10 @@ stopifnot(length(grep("'epsilon' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check that length of upsilon equals 1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = c(0.0357, 0.0357),
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -603,10 +683,10 @@ stopifnot(length(grep("'upsilon' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of gamma equals 1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = c(0.1, 0.1),
                                alpha   = 1.0,
@@ -623,10 +703,10 @@ stopifnot(length(grep("'gamma' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of alpha equals 1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = c(1.0, 1.0),
@@ -643,10 +723,10 @@ stopifnot(length(grep("'alpha' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of beta_t1 equals 1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -663,10 +743,10 @@ stopifnot(length(grep("'beta_t1' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of beta_t2 equals 1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -683,10 +763,10 @@ stopifnot(length(grep("'beta_t2' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of beta_t3 equals 1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -703,10 +783,10 @@ stopifnot(length(grep("'beta_t3' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of beta_t4 equals 1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -722,11 +802,11 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'beta_t4' must be of length 1",
                       res[[1]]$message)) > 0)
 
-## Check that length of end_t1 equals 1 or nrow(init)
-res <- tools::assertError(SISe(init    = init,
+## Check that length of end_t1 equals 1 or nrow(u0)
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -739,14 +819,14 @@ res <- tools::assertError(SISe(init    = init,
                                end_t3  = 273,
                                end_t4  = 365,
                                epsilon = 0.000011))
-stopifnot(length(grep("'end_t1' must be of length 1 or 'nrow\\(init\\)'",
+stopifnot(length(grep("'end_t1' must be of length 1 or 'nrow\\(u0\\)'",
                       res[[1]]$message)) > 0)
 
-## Check that length of end_t2 equals 1 or nrow(init)
-res <- tools::assertError(SISe(init    = init,
+## Check that length of end_t2 equals 1 or nrow(u0)
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -759,14 +839,14 @@ res <- tools::assertError(SISe(init    = init,
                                end_t3  = 273,
                                end_t4  = 365,
                                epsilon = 0.000011))
-stopifnot(length(grep("'end_t2' must be of length 1 or 'nrow\\(init\\)'",
+stopifnot(length(grep("'end_t2' must be of length 1 or 'nrow\\(u0\\)'",
                       res[[1]]$message)) > 0)
 
-## Check that length of end_t3 equals 1 or nrow(init)
-res <- tools::assertError(SISe(init    = init,
+## Check that length of end_t3 equals 1 or nrow(u0)
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -779,14 +859,14 @@ res <- tools::assertError(SISe(init    = init,
                                end_t3  = c(273, 273),
                                end_t4  = 365,
                                epsilon = 0.000011))
-stopifnot(length(grep("'end_t3' must be of length 1 or 'nrow\\(init\\)'",
+stopifnot(length(grep("'end_t3' must be of length 1 or 'nrow\\(u0\\)'",
                       res[[1]]$message)) > 0)
 
-## Check that length of end_t4 equals 1 or nrow(init)
-res <- tools::assertError(SISe(init    = init,
+## Check that length of end_t4 equals 1 or nrow(u0)
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -799,14 +879,14 @@ res <- tools::assertError(SISe(init    = init,
                                end_t3  = 273,
                                end_t4  = c(365, 365),
                                epsilon = 0.000011))
-stopifnot(length(grep("'end_t4' must be of length 1 or 'nrow\\(init\\)'",
+stopifnot(length(grep("'end_t4' must be of length 1 or 'nrow\\(u0\\)'",
                       res[[1]]$message)) > 0)
 
 ## Check that length of epsilon equals 1
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -823,10 +903,10 @@ stopifnot(length(grep("'epsilon' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check interval endpoints
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -842,10 +922,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t1' must be greater than or equal to '0'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -861,10 +941,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t1' must be less than 'end_t2'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -880,10 +960,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t2' must be less than 'end_t3'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -899,10 +979,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t3' must be less than '364'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -918,10 +998,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t4' must be greater than or equal to '0'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -937,10 +1017,10 @@ res <- tools::assertError(SISe(init    = init,
 stopifnot(length(grep("'end_t4' must be less than or equal to '365'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe(init    = init,
+res <- tools::assertError(SISe(u0      = u0,
                                tspan   = seq_len(10) - 1,
                                events  = NULL,
-                               phi     = rep(1, nrow(init)),
+                               phi     = rep(1, nrow(u0)),
                                upsilon = 0.0357,
                                gamma   = 0.1,
                                alpha   = 1.0,
@@ -958,10 +1038,10 @@ stopifnot(length(grep(
     res[[1]]$message)) > 0)
 
 ## Check 'suscpetible' and 'infected' methods
-model <- SISe(init    = init,
+model <- SISe(u0      = u0,
               tspan   = seq_len(10) - 1,
               events  = NULL,
-              phi     = rep(0, nrow(init)),
+              phi     = rep(0, nrow(u0)),
               upsilon = 0.0357,
               gamma   = 0.1,
               alpha   = 1.0,
@@ -1008,15 +1088,30 @@ stopifnot(file.exists(pdf_file))
 unlink(pdf_file)
 
 ## Check that C SISe run function fails for misspecified SISe model
-res <- tools::assertError(.Call(SimInf:::SISe_run, NULL, NULL, NULL))
-stopifnot(length(grep("Invalid SISe model",
-                      res[[1]]$message)) > 0)
-res <- tools::assertError(.Call(SimInf:::SISe_run, "SISe", NULL, NULL))
-stopifnot(length(grep("Invalid SISe model",
-                      res[[1]]$message)) > 0)
+res <- .Call("SISe_run", NULL, NULL, NULL, PACKAGE = "SimInf")
+stopifnot(identical(res$error, -10L))
 
-setClass("DummySISe", slots = c(a = "character"))
-model <- new("DummySISe", a = "SISe")
-res <- tools::assertError(.Call(SimInf:::SISe_run, model, NULL, NULL))
-stopifnot(length(grep("Invalid SISe model: DummySISe",
+res <- .Call("SISe_run", "SISe", NULL, NULL, PACKAGE = "SimInf")
+stopifnot(identical(res$error, -10L))
+
+## Check error non-finite v
+model <- SISe(u0      = u0,
+              tspan   = seq_len(10) - 1,
+              events  = NULL,
+              phi     = rep(1, nrow(u0)),
+              upsilon = 0.0357,
+              gamma   = 0.1,
+              alpha   = 1.0,
+              beta_t1 = 0.19,
+              beta_t2 = 0.085,
+              beta_t3 = 0.075,
+              beta_t4 = 0.185,
+              end_t1  = 91,
+              end_t2  = 182,
+              end_t3  = 273,
+              end_t4  = 365,
+              epsilon = 0.000011)
+model@gdata <- rep(Inf, length(model@gdata))
+res <- tools::assertError(run(model))
+stopifnot(length(grep("The continuous state 'v' is not finite.",
                       res[[1]]$message)) > 0)
