@@ -1,7 +1,7 @@
 ## SimInf, a framework for stochastic disease spread simulations
 ## Copyright (C) 2015  Pavol Bauer
-## Copyright (C) 2015  Stefan Engblom
-## Copyright (C) 2015  Stefan Widgren
+## Copyright (C) 2015 - 2016  Stefan Engblom
+## Copyright (C) 2015 - 2016  Stefan Widgren
 ##
 ## This program is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -21,54 +21,53 @@ library(SimInf)
 ## For debugging
 sessionInfo()
 
-init <- structure(list(id  = c(0, 1, 2, 3, 4, 5),
-                       S_1 = c(0, 1, 2, 3, 4, 5),
-                       I_1 = c(0, 0, 0, 0, 0, 0),
-                       S_2 = c(0, 1, 2, 3, 4, 5),
-                       I_2 = c(0, 0, 0, 0, 0, 0),
-                       S_3 = c(0, 1, 2, 3, 4, 5),
-                       I_3 = c(0, 0, 0, 0, 0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -6L), class = "data.frame")
+## Check invalid u0
+res <- tools::assertError(SISe3(u0 = "u0"))
+stopifnot(length(grep("'u0' must be a data.frame",
+                      res[[1]]$message)) > 0)
 
-## Check missing columns in init
+u0 <- structure(list(S_1 = c(0, 1, 2, 3, 4, 5),
+                     I_1 = c(0, 0, 0, 0, 0, 0),
+                     S_2 = c(0, 1, 2, 3, 4, 5),
+                     I_2 = c(0, 0, 0, 0, 0, 0),
+                     S_3 = c(0, 1, 2, 3, 4, 5),
+                     I_3 = c(0, 0, 0, 0, 0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -6L), class = "data.frame")
+
+## Check missing columns in u0
 res <- tools::assertError(
-    SISe3(init = init[, c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3")]))
-stopifnot(length(grep("Missing columns in init",
+    SISe3(u0 = u0[, c("I_1", "S_2", "I_2", "S_3", "I_3")]))
+stopifnot(length(grep("Missing columns in u0",
                       res[[1]]$message)) > 0)
 
 res <- tools::assertError(
-    SISe3(init = init[, c("id", "I_1", "S_2", "I_2", "S_3", "I_3")]))
-stopifnot(length(grep("Missing columns in init",
+    SISe3(u0 = u0[, c("S_1", "S_2", "I_2", "S_3", "I_3")]))
+stopifnot(length(grep("Missing columns in u0",
                       res[[1]]$message)) > 0)
 
 res <- tools::assertError(
-    SISe3(init = init[, c("id", "S_1", "S_2", "I_2", "S_3", "I_3")]))
-stopifnot(length(grep("Missing columns in init",
+    SISe3(u0 = u0[, c("S_1", "I_1", "I_2", "S_3", "I_3")]))
+stopifnot(length(grep("Missing columns in u0",
                       res[[1]]$message)) > 0)
 
 res <- tools::assertError(
-    SISe3(init = init[, c("id", "S_1", "I_1", "I_2", "S_3", "I_3")]))
-stopifnot(length(grep("Missing columns in init",
+    SISe3(u0 = u0[, c("S_1", "I_1", "S_2", "S_3", "I_3")]))
+stopifnot(length(grep("Missing columns in u0",
                       res[[1]]$message)) > 0)
 
 res <- tools::assertError(
-    SISe3(init = init[, c("id", "S_1", "I_1", "S_2", "S_3", "I_3")]))
-stopifnot(length(grep("Missing columns in init",
+    SISe3(u0 = u0[, c("S_1", "I_1", "S_2", "I_2", "I_3")]))
+stopifnot(length(grep("Missing columns in u0",
                       res[[1]]$message)) > 0)
 
 res <- tools::assertError(
-    SISe3(init = init[, c("id", "S_1", "I_1", "S_2", "I_2", "I_3")]))
-stopifnot(length(grep("Missing columns in init",
-                      res[[1]]$message)) > 0)
-
-res <- tools::assertError(
-    SISe3(init = init[, c("id", "S_1", "I_1", "S_2", "I_2", "S_3")]))
-stopifnot(length(grep("Missing columns in init",
+    SISe3(u0 = u0[, c("S_1", "I_1", "S_2", "I_2", "S_3")]))
+stopifnot(length(grep("Missing columns in u0",
                       res[[1]]$message)) > 0)
 
 ## Check missing upsilon_1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -91,7 +90,7 @@ stopifnot(length(grep("'upsilon_1' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing upsilon_2
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -114,7 +113,7 @@ stopifnot(length(grep("'upsilon_2' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing upsilon_3
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -137,7 +136,7 @@ stopifnot(length(grep("'upsilon_3' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing gamma_1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -160,7 +159,7 @@ stopifnot(length(grep("'gamma_1' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing gamma_2
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -183,7 +182,7 @@ stopifnot(length(grep("'gamma_2' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing gamma_3
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -206,7 +205,7 @@ stopifnot(length(grep("'gamma_3' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing alpha
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -229,7 +228,7 @@ stopifnot(length(grep("'alpha' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing beta_t1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -252,7 +251,7 @@ stopifnot(length(grep("'beta_t1' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing beta_t2
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -275,7 +274,7 @@ stopifnot(length(grep("'beta_t2' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing beta_t3
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -298,7 +297,7 @@ stopifnot(length(grep("'beta_t3' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing beta_t4
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -321,7 +320,7 @@ stopifnot(length(grep("'beta_t4' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing end_t1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -344,7 +343,7 @@ stopifnot(length(grep("'end_t1' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing end_t2
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -367,7 +366,7 @@ stopifnot(length(grep("'end_t2' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing end_t3
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -390,7 +389,7 @@ stopifnot(length(grep("'end_t3' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing end_t4
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -413,7 +412,7 @@ stopifnot(length(grep("'end_t4' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check missing epsilon
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -436,7 +435,7 @@ stopifnot(length(grep("'epsilon' is missing",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric upsilon_1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -460,7 +459,7 @@ stopifnot(length(grep("'upsilon_1' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric upsilon_2
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -484,7 +483,7 @@ stopifnot(length(grep("'upsilon_2' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric upsilon_3
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -508,7 +507,7 @@ stopifnot(length(grep("'upsilon_3' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric gamma_1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -532,7 +531,7 @@ stopifnot(length(grep("'gamma_1' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric gamma_2
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -556,7 +555,7 @@ stopifnot(length(grep("'gamma_2' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric gamma_3
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -580,7 +579,7 @@ stopifnot(length(grep("'gamma_3' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric alpha
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -604,7 +603,7 @@ stopifnot(length(grep("'alpha' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric beta_t1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -628,7 +627,7 @@ stopifnot(length(grep("'beta_t1' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric beta_t2
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -652,7 +651,7 @@ stopifnot(length(grep("'beta_t2' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric beta_t3
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -676,7 +675,7 @@ stopifnot(length(grep("'beta_t3' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric beta_t4
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -700,7 +699,7 @@ stopifnot(length(grep("'beta_t4' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check non-integer end_t1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -723,7 +722,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t1' must be integer",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -747,7 +746,7 @@ stopifnot(length(grep("'end_t1' must be integer",
                       res[[1]]$message)) > 0)
 
 ## Check non-integer end_t2
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -770,7 +769,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t2' must be integer",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -794,7 +793,7 @@ stopifnot(length(grep("'end_t2' must be integer",
                       res[[1]]$message)) > 0)
 
 ## Check non-integer end_t3
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -817,7 +816,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t3' must be integer",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -841,7 +840,7 @@ stopifnot(length(grep("'end_t3' must be integer",
                       res[[1]]$message)) > 0)
 
 ## Check non-integer end_t4
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -864,7 +863,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t4' must be integer",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -888,7 +887,7 @@ stopifnot(length(grep("'end_t4' must be integer",
                       res[[1]]$message)) > 0)
 
 ## Check non-numeric epsilon
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -912,7 +911,7 @@ stopifnot(length(grep("'epsilon' must be numeric",
                       res[[1]]$message)) > 0)
 
 ## Check that length of upsilon_1 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -936,7 +935,7 @@ stopifnot(length(grep("'upsilon_1' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of upsilon_2 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -960,7 +959,7 @@ stopifnot(length(grep("'upsilon_2' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of upsilon_3 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -984,7 +983,7 @@ stopifnot(length(grep("'upsilon_3' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of gamma_1 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1008,7 +1007,7 @@ stopifnot(length(grep("'gamma_1' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of gamma_2 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1032,7 +1031,7 @@ stopifnot(length(grep("'gamma_2' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of gamma_3 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1056,7 +1055,7 @@ stopifnot(length(grep("'gamma_3' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of alpha equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1080,7 +1079,7 @@ stopifnot(length(grep("'alpha' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of beta_t1 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1104,7 +1103,7 @@ stopifnot(length(grep("'beta_t1' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of beta_t2 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1128,7 +1127,7 @@ stopifnot(length(grep("'beta_t2' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of beta_t3 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1152,7 +1151,7 @@ stopifnot(length(grep("'beta_t3' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check that length of beta_t4 equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1175,8 +1174,8 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'beta_t4' must be of length 1",
                       res[[1]]$message)) > 0)
 
-## Check that length of end_t1 equals 1 or nrow(init)
-res <- tools::assertError(SISe3(init      = init,
+## Check that length of end_t1 equals 1 or nrow(u0)
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1196,11 +1195,11 @@ res <- tools::assertError(SISe3(init      = init,
                                 end_t3    = 273,
                                 end_t4    = 365,
                                 epsilon   = 0.000011))
-stopifnot(length(grep("'end_t1' must be of length 1 or 'nrow\\(init\\)'",
+stopifnot(length(grep("'end_t1' must be of length 1 or 'nrow\\(u0\\)'",
                       res[[1]]$message)) > 0)
 
-## Check that length of end_t2 equals 1 or nrow(init)
-res <- tools::assertError(SISe3(init      = init,
+## Check that length of end_t2 equals 1 or nrow(u0)
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1220,11 +1219,11 @@ res <- tools::assertError(SISe3(init      = init,
                                 end_t3    = 273,
                                 end_t4    = 365,
                                 epsilon   = 0.000011))
-stopifnot(length(grep("'end_t2' must be of length 1 or 'nrow\\(init\\)'",
+stopifnot(length(grep("'end_t2' must be of length 1 or 'nrow\\(u0\\)'",
                       res[[1]]$message)) > 0)
 
-## Check that length of end_t3 equals 1 or nrow(init)
-res <- tools::assertError(SISe3(init      = init,
+## Check that length of end_t3 equals 1 or nrow(u0)
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1244,11 +1243,11 @@ res <- tools::assertError(SISe3(init      = init,
                                 end_t3    = c(273, 273),
                                 end_t4    = 365,
                                 epsilon   = 0.000011))
-stopifnot(length(grep("'end_t3' must be of length 1 or 'nrow\\(init\\)'",
+stopifnot(length(grep("'end_t3' must be of length 1 or 'nrow\\(u0\\)'",
                       res[[1]]$message)) > 0)
 
-## Check that length of end_t4 equals 1 or nrow(init)
-res <- tools::assertError(SISe3(init      = init,
+## Check that length of end_t4 equals 1 or nrow(u0)
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1268,11 +1267,11 @@ res <- tools::assertError(SISe3(init      = init,
                                 end_t3    = 273,
                                 end_t4    = c(365, 365),
                                 epsilon   = 0.000011))
-stopifnot(length(grep("'end_t4' must be of length 1 or 'nrow\\(init\\)'",
+stopifnot(length(grep("'end_t4' must be of length 1 or 'nrow\\(u0\\)'",
                       res[[1]]$message)) > 0)
 
 ## Check that length of epsilon equals 1
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1296,7 +1295,7 @@ stopifnot(length(grep("'epsilon' must be of length 1",
                       res[[1]]$message)) > 0)
 
 ## Check interval endpoints
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1319,7 +1318,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t1' must be greater than or equal to '0'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1342,7 +1341,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t1' must be less than 'end_t2'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1365,7 +1364,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t2' must be less than 'end_t3'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1388,7 +1387,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t3' must be less than '364'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1411,7 +1410,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t4' must be greater than or equal to '0'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1434,7 +1433,7 @@ res <- tools::assertError(SISe3(init      = init,
 stopifnot(length(grep("'end_t4' must be less than or equal to '365'",
                       res[[1]]$message)) > 0)
 
-res <- tools::assertError(SISe3(init      = init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = seq_len(10) - 1,
                                 events    = NULL,
                                 phi       = rep(1, 6),
@@ -1459,7 +1458,7 @@ stopifnot(length(grep(
     res[[1]]$message)) > 0)
 
 ## Check 'suscpetible' and 'infected' methods
-model <- SISe3(init      = init,
+model <- SISe3(u0        = u0,
                tspan     = seq_len(10) - 1,
                events    = NULL,
                phi       = rep(0, 6),
@@ -1513,15 +1512,34 @@ stopifnot(file.exists(pdf_file))
 unlink(pdf_file)
 
 ## Check that C SISe3 run function fails for misspecified SISe3 model
-res <- tools::assertError(.Call(SimInf:::SISe3_run, NULL, NULL, NULL))
-stopifnot(length(grep("Invalid SISe3 model",
-                      res[[1]]$message)) > 0)
-res <- tools::assertError(.Call(SimInf:::SISe3_run, "SISe3", NULL, NULL))
-stopifnot(length(grep("Invalid SISe3 model",
-                      res[[1]]$message)) > 0)
+res <- .Call("SISe3_run", NULL, NULL, NULL, PACKAGE = "SimInf")
+stopifnot(identical(res$error, -10L))
 
-setClass("DummySISe3", slots = c(a = "character"))
-model <- new("DummySISe3", a = "SISe3")
-res <- tools::assertError(.Call(SimInf:::SISe3_run, model, NULL, NULL))
-stopifnot(length(grep("Invalid SISe3 model: DummySISe3",
+res <- .Call("SISe3_run", "SISe3", NULL, NULL, PACKAGE = "SimInf")
+stopifnot(identical(res$error, -10L))
+
+## Check error non-finite v
+model <- SISe3(u0        = u0,
+               tspan     = seq_len(10) - 1,
+               events    = NULL,
+               phi       = rep(1, 6),
+               upsilon_1 = 0.0357,
+               upsilon_2 = 0.0357,
+               upsilon_3 = 0.00935,
+               gamma_1   = 0.1,
+               gamma_2   = 0.1,
+               gamma_3   = 0.1,
+               alpha     = 1.0,
+               beta_t1   = 0.19,
+               beta_t2   = 0.085,
+               beta_t3   = 0.075,
+               beta_t4   = 0.185,
+               end_t1    = 91,
+               end_t2    = 182,
+               end_t3    = 273,
+               end_t4    = 365,
+               epsilon   = 0.000011)
+model@gdata <- rep(Inf, length(model@gdata))
+res <- tools::assertError(run(model))
+stopifnot(length(grep("The continuous state 'v' is not finite.",
                       res[[1]]$message)) > 0)
