@@ -1,7 +1,7 @@
 ## SimInf, a framework for stochastic disease spread simulations
 ## Copyright (C) 2015  Pavol Bauer
-## Copyright (C) 2015  Stefan Engblom
-## Copyright (C) 2015  Stefan Widgren
+## Copyright (C) 2015 - 2016  Stefan Engblom
+## Copyright (C) 2015 - 2016  Stefan Widgren
 ##
 ## This program is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -27,34 +27,33 @@ sessionInfo()
 ## 3 Age categories
 ## 2 Disease-states: Susceptible & Infected
 ##
-## One individual start in susceptible state in node = 1, with a zero
+## One individual start in susceptible state in node = 2, with a zero
 ## probability of becoming infected.
 ##
-## At t = 1, two individuals are moved to node = 0. This should fail.
-init <- structure(list(id  = c(0, 1),
-                       S_1 = c(0, 1),
-                       I_1 = c(0, 0),
-                       S_2 = c(0, 0),
-                       I_2 = c(0, 0),
-                       S_3 = c(0, 0),
-                       I_3 = c(0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -2L),
-                  class = "data.frame")
+## At t = 1, two individuals are moved to node = 1. This should fail.
+u0 <- structure(list(S_1 = c(0, 1),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 0),
+                     I_3 = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L),
+                class = "data.frame")
 
 events <- structure(list(event      = 3,
                          time       = 1,
-                         node       = 1,
-                         dest       = 0,
+                         node       = 2,
+                         dest       = 1,
                          n          = 2,
                          proportion = 1,
-                         select     = 3,
-                         shift      = -1),
+                         select     = 4,
+                         shift      = 0),
                     .Names = c("event", "time", "node", "dest",
                         "n", "proportion", "select", "shift"),
                     row.names = c(NA, -1L), class = "data.frame")
 
-model <- SISe3(init,
+model <- SISe3(u0        = u0,
                tspan     = 0:10,
                events    = events,
                phi       = rep(0, 2),
@@ -76,12 +75,12 @@ model <- SISe3(init,
                epsilon   = 0)
 
 res <- tools::assertError(run(model, threads = 1))
-stopifnot(length(grep("Negative state detected.",
+stopifnot(length(grep("Unable to sample individuals for event.",
                       res[[1]]$message)) > 0)
 
 if (SimInf:::have_openmp()) {
     res <- tools::assertError(run(model, threads = 2))
-    stopifnot(length(grep("Negative state detected.",
+    stopifnot(length(grep("Unable to sample individuals for event.",
                           res[[1]]$message)) > 0)
 }
 
@@ -89,33 +88,32 @@ if (SimInf:::have_openmp()) {
 ## 3 Age categories
 ## 2 Disease-states: Susceptible & Infected
 ##
-## One individual start in susceptible state in node = 1, with a zero
+## One individual start in susceptible state in node = 2, with a zero
 ## probability of becoming infected.
 ##
-## At t = 1, -1 individuals are moved to node = 0. This should fail.
-init <- structure(list(id  = c(0, 1),
-                       S_1 = c(0, 1),
-                       I_1 = c(0, 0),
-                       S_2 = c(0, 0),
-                       I_2 = c(0, 0),
-                       S_3 = c(0, 0),
-                       I_3 = c(0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -2L), class = "data.frame")
+## At t = 1, -1 individuals are moved to node = 1. This should fail.
+u0 <- structure(list(S_1 = c(0, 1),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 0),
+                     I_3 = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L), class = "data.frame")
 
 events <- structure(list(event      = 3,
                          time       = 1,
-                         node       = 1,
-                         dest       = 0,
+                         node       = 2,
+                         dest       = 1,
                          n          = -1,
                          proportion = 1,
-                         select     = 3,
-                         shift      = -1),
+                         select     = 4,
+                         shift      = 0),
                     .Names = c("event", "time", "node", "dest",
                         "n", "proportion", "select", "shift"),
                     row.names = c(NA, -1L), class = "data.frame")
 
-model <- SISe3(init,
+model <- SISe3(u0        = u0,
                tspan     = 0:10,
                events    = events,
                phi       = rep(0, 2),
@@ -137,12 +135,12 @@ model <- SISe3(init,
                epsilon   = 0)
 
 res <- tools::assertError(run(model, threads = 1))
-stopifnot(length(grep("Negative state detected.",
+stopifnot(length(grep("Unable to sample individuals for event.",
                       res[[1]]$message)) > 0)
 
 if (SimInf:::have_openmp()) {
     res <- tools::assertError(run(model, threads = 2))
-    stopifnot(length(grep("Negative state detected.",
+    stopifnot(length(grep("Unable to sample individuals for event.",
                           res[[1]]$message)) > 0)
 }
 
@@ -150,35 +148,34 @@ if (SimInf:::have_openmp()) {
 ## 3 Age categories
 ## 2 Disease-states: Susceptible & Infected
 ##
-## One individual start in susceptible state in node = 1, with a zero
+## One individual start in susceptible state in node = 2, with a zero
 ## probability of becoming infected.
 ##
-## At t = 1, a proportion of 10 individuals are moved to node =
-## 0. This should fail.
-init <- structure(list(id  = c(0, 1),
-                       S_1 = c(0, 1),
-                       I_1 = c(0, 0),
-                       S_2 = c(0, 0),
-                       I_2 = c(0, 0),
-                       S_3 = c(0, 0),
-                       I_3 = c(0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -2L), class = "data.frame")
+## At t = 1, a proportion of 10 individuals are moved to node = 1.
+## This should fail.
+u0 <- structure(list(S_1 = c(0, 1),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 0),
+                     I_3 = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L), class = "data.frame")
 
 events <- structure(list(event      = 3,
                          time       = 1,
-                         node       = 1,
-                         dest       = 0,
+                         node       = 2,
+                         dest       = 1,
                          n          = 0,
                          proportion = 10,
-                         select     = 3,
-                         shift      = -1),
+                         select     = 4,
+                         shift      = 0),
                     .Names = c("event", "time", "node", "dest",
                         "n", "proportion", "select", "shift"),
                     row.names = c(NA, -1L), class = "data.frame")
 
 ## We should not be able to create model with prop = 10
-res <- tools::assertError(SISe3(init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = 0:10,
                                 events    = events,
                                 phi       = rep(0, 2),
@@ -204,7 +201,7 @@ stopifnot(length(grep("prop must be in the range 0 <= prop <= 1",
 ## Replace proportion = 10 to proportion = 1
 events$proportion <- 1
 
-model <- SISe3(init,
+model <- SISe3(u0        = u0,
                tspan     = 0:10,
                events    = events,
                phi       = rep(0, 2),
@@ -225,16 +222,18 @@ model <- SISe3(init,
                end_t4    = 365,
                epsilon   = 0)
 
-## Replace proportion = 10 to proportion = 1
+## Replace proportion = 1 with proportion = 10
 model@events@proportion <- 10
 
-res <- tools::assertError(run(model, threads = 1))
-stopifnot(length(grep("Negative state detected.",
+res <- .Call("SISe3_run", model, 1L, NULL, PACKAGE = "SimInf")
+res <- tools::assertError(SimInf:::siminf_error(res$error))
+stopifnot(length(grep("Unable to sample individuals for event.",
                       res[[1]]$message)) > 0)
 
 if (SimInf:::have_openmp()) {
-    res <- tools::assertError(run(model, threads = 2))
-    stopifnot(length(grep("Negative state detected.",
+    res <- .Call("SISe3_run", model, 2L, NULL, PACKAGE = "SimInf")
+    res <- tools::assertError(SimInf:::siminf_error(res$error))
+    stopifnot(length(grep("Unable to sample individuals for event.",
                           res[[1]]$message)) > 0)
 }
 
@@ -242,35 +241,34 @@ if (SimInf:::have_openmp()) {
 ## 3 Age categories
 ## 2 Disease-states: Susceptible & Infected
 ##
-## One individual start in susceptible state in node = 1, with a zero
+## One individual start in susceptible state in node = 2, with a zero
 ## probability of becoming infected.
 ##
-## At t = 1, a proportion of -1 individuals are moved to node =
-## 0. This should fail.
-init <- structure(list(id  = c(0, 1),
-                       S_1 = c(0, 1),
-                       I_1 = c(0, 0),
-                       S_2 = c(0, 0),
-                       I_2 = c(0, 0),
-                       S_3 = c(0, 0),
-                       I_3 = c(0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -2L), class = "data.frame")
+## At t = 1, a proportion of -1 individuals are moved to node = 1.
+## This should fail.
+u0 <- structure(list(S_1 = c(0, 1),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 0),
+                     I_3 = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L), class = "data.frame")
 
 events <- structure(list(event      = 3,
                          time       = 1,
-                         node       = 1,
-                         dest       = 0,
+                         node       = 2,
+                         dest       = 1,
                          n          = 0,
                          proportion = -1,
-                         select     = 3,
-                         shift      = -1),
+                         select     = 4,
+                         shift      = 0),
                     .Names = c("event", "time", "node", "dest",
                         "n", "proportion", "select", "shift"),
                     row.names = c(NA, -1L), class = "data.frame")
 
 ## We should not be able to create model with proportion = -1
-res <- tools::assertError(SISe3(init,
+res <- tools::assertError(SISe3(u0        = u0,
                                 tspan     = 0:10,
                                 events    = events,
                                 phi       = rep(0, 2),
@@ -293,10 +291,10 @@ res <- tools::assertError(SISe3(init,
 stopifnot(length(grep("prop must be in the range 0 <= prop <= 1",
                       res[[1]]$message)) > 0)
 
-## Replace proportion = -1 to proportion = 0
+## Replace proportion = -1 with proportion = 0
 events$proportion <- 0
 
-model <- SISe3(init,
+model <- SISe3(u0        = u0,
                tspan     = 0:10,
                events    = events,
                phi       = rep(0, 2),
@@ -317,16 +315,18 @@ model <- SISe3(init,
                end_t4    = 365,
                epsilon   = 0)
 
-## Replace proportion = 0 to proportion = -1
+## Replace proportion = 0 with proportion = -1
 model@events@proportion <- -1
 
-res <- tools::assertError(run(model, threads = 1))
-stopifnot(length(grep("Negative state detected.",
+res <- .Call("SISe3_run", model, 1L, NULL, PACKAGE = "SimInf")
+res <- tools::assertError(SimInf:::siminf_error(res$error))
+stopifnot(length(grep("Unable to sample individuals for event.",
                       res[[1]]$message)) > 0)
 
 if (SimInf:::have_openmp()) {
-    res <- tools::assertError(run(model, threads = 2))
-    stopifnot(length(grep("Negative state detected.",
+    res <- .Call("SISe3_run", model, 2L, NULL, PACKAGE = "SimInf")
+    res <- tools::assertError(SimInf:::siminf_error(res$error))
+    stopifnot(length(grep("Unable to sample individuals for event.",
                           res[[1]]$message)) > 0)
 }
 
@@ -334,33 +334,32 @@ if (SimInf:::have_openmp()) {
 ## 3 Age categories
 ## 2 Disease-states: Susceptible & Infected
 ##
-## One individual start in susceptible state in node = 1, with a zero
+## One individual start in susceptible state in node = 2, with a zero
 ## probability of becoming infected.
 ##
-## At t = 1, a proportion of 0 individuals are moved to node = 0.
-init <- structure(list(id  = c(0, 1),
-                       S_1 = c(0, 1),
-                       I_1 = c(0, 0),
-                       S_2 = c(0, 0),
-                       I_2 = c(0, 0),
-                       S_3 = c(0, 0),
-                       I_3  = c(0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -2L), class = "data.frame")
+## At t = 1, a proportion of 0 individuals are moved to node = 1.
+u0 <- structure(list(S_1 = c(0, 1),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 0),
+                     I_3  = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L), class = "data.frame")
 
 events <- structure(list(event      = 3,
                          time       = 1,
-                         node       = 1,
-                         dest       = 0,
+                         node       = 2,
+                         dest       = 1,
                          n          = 0,
                          proportion = 0,
-                         select     = 3,
-                         shift      = -1),
+                         select     = 4,
+                         shift      = 0),
                     .Names = c("event", "time", "node", "dest",
                         "n", "proportion", "select", "shift"),
                     row.names = c(NA, -1L), class = "data.frame")
 
-model <- SISe3(init,
+model <- SISe3(u0        = u0,
                tspan     = 0:2,
                events    = events,
                phi       = rep(0, 2),
@@ -411,33 +410,32 @@ if (SimInf:::have_openmp()) {
 ## 3 Age categories
 ## 2 Disease-states: Susceptible & Infected
 ##
-## One individual start in susceptible state in node = 1, with a zero
+## One individual start in susceptible state in node = 2, with a zero
 ## probability of becoming infected.
 ##
-## At t = 1, proportion of all (1) individuals are moved to node = 0.
-init <- structure(list(id  = c(0, 1),
-                       S_1 = c(0, 1),
-                       I_1 = c(0, 0),
-                       S_2 = c(0, 0),
-                       I_2 = c(0, 0),
-                       S_3  = c(0, 0),
-                       I_3  = c(0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -2L), class = "data.frame")
+## At t = 1, proportion of all (1) individuals are moved to node = 1.
+u0 <- structure(list(S_1 = c(0, 1),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3  = c(0, 0),
+                     I_3  = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L), class = "data.frame")
 
 events <- structure(list(event      = 3,
                          time       = 1,
-                         node       = 1,
-                         dest       = 0,
+                         node       = 2,
+                         dest       = 1,
                          n          = 1,
                          proportion = 0,
-                         select     = 3,
-                         shift      = -1),
+                         select     = 4,
+                         shift      = 0),
                     .Names = c("event", "time", "node", "dest",
                         "n", "proportion", "select", "shift"),
                     row.names = c(NA, -1L), class = "data.frame")
 
-model <- SISe3(init,
+model <- SISe3(u0        = u0,
                tspan     = 0:2,
                events    = events,
                phi       = rep(0, 2),
@@ -488,33 +486,32 @@ if (SimInf:::have_openmp()) {
 ## 3 Age categories
 ## 2 Disease-states: Susceptible & Infected
 ##
-## Two individuals start in susceptible state in node = 1, with a zero
+## Two individuals start in susceptible state in node = 2, with a zero
 ## probability of becoming infected.
 ##
-## At t = 1, Nkind = 1, one individual is moved to node = 0.
-init <- structure(list(id  = c(0, 1),
-                       S_1 = c(0, 2),
-                       I_1 = c(0, 0),
-                       S_2 = c(0, 0),
-                       I_2 = c(0, 0),
-                       S_3 = c(0, 0),
-                       I_3  = c(0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -2L), class = "data.frame")
+## At t = 1, Nkind = 1, one individual is moved to node = 1.
+u0 <- structure(list(S_1 = c(0, 2),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 0),
+                     I_3  = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L), class = "data.frame")
 
 events <- structure(list(event      = 3,
                          time       = 1,
-                         node       = 1,
-                         dest       = 0,
+                         node       = 2,
+                         dest       = 1,
                          n          = 1,
                          proportion = 0,
-                         select     = 3,
-                         shift      = -1),
+                         select     = 4,
+                         shift      = 0),
                     .Names = c("event", "time", "node", "dest",
                         "n", "proportion", "select", "shift"),
                     row.names = c(NA, -1L), class = "data.frame")
 
-model <- SISe3(init,
+model <- SISe3(u0        = u0,
                tspan     = 0:2,
                events    = events,
                phi       = rep(0, 2),
@@ -566,33 +563,32 @@ if (SimInf:::have_openmp()) {
 ## 2 Disease-states: Susceptible & Infected
 ##
 ## Two individuals start in susceptible state and 8 individuals in
-## infected state in node = 1, with a zero probability of becoming
+## infected state in node = 2, with a zero probability of becoming
 ## infected.
 ##
-## At t = 1, Nkind = 2, one individual is moved to node = 0.
-init <- structure(list(id  = c(0, 1),
-                       S_1 = c(0, 2),
-                       I_1 = c(0, 8),
-                       S_2 = c(0, 0),
-                       I_2 = c(0, 0),
-                       S_3 = c(0, 0),
-                       I_3 = c(0, 0)),
-                  .Names = c("id", "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                  row.names = c(NA, -2L), class = "data.frame")
+## At t = 1, Nkind = 2, one individual is moved to node = 1.
+u0 <- structure(list(S_1 = c(0, 2),
+                     I_1 = c(0, 8),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 0),
+                     I_3 = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L), class = "data.frame")
 
 events <- structure(list(event      = 3,
                          time       = 1,
-                         node       = 1,
-                         dest       = 0,
+                         node       = 2,
+                         dest       = 1,
                          n          = 1,
                          proportion = 0,
-                         select     = 3,
-                         shift      = -1),
+                         select     = 4,
+                         shift      = 0),
                     .Names = c("event", "time", "node", "dest",
                         "n", "proportion", "select", "shift"),
                     row.names = c(NA, -1L), class = "data.frame")
 
-model <- SISe3(init,
+model <- SISe3(u0        = u0,
                tspan     = 0:2,
                events    = events,
                phi       = rep(0, 2),
@@ -638,3 +634,144 @@ if (SimInf:::have_openmp()) {
     stopifnot(identical(model@u0, result_omp@u0))
     stopifnot(identical(model@events, result_omp@events))
 }
+
+## 2 Nodes
+## 3 Age categories
+## 2 Disease-states: Susceptible & Infected
+##
+## Two individuals start in susceptible state in node = 2, with a zero
+## probability of becoming infected.
+##
+## At t = 1, one individual is moved to node = 1 using a select matrix
+## that can only select a susceptible from S_1.
+u0 <- structure(list(S_1 = c(0, 2),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 0),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 0),
+                     I_3 = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L),
+                class = "data.frame")
+
+events <- structure(list(event      = 3,
+                         time       = 1,
+                         node       = 2,
+                         dest       = 1,
+                         n          = 1,
+                         proportion = 0,
+                         select     = 1,
+                         shift      = 0),
+                    .Names = c("event", "time", "node", "dest",
+                        "n", "proportion", "select", "shift"),
+                    row.names = c(NA, -1L), class = "data.frame")
+
+model <- SISe3(u0        = u0,
+               tspan     = 0:10,
+               events    = events,
+               phi       = rep(0, 2),
+               upsilon_1 = 0,
+               upsilon_2 = 0,
+               upsilon_3 = 0,
+               gamma_1   = 1,
+               gamma_2   = 1,
+               gamma_3   = 1,
+               alpha     = 0,
+               beta_t1   = 1,
+               beta_t2   = 1,
+               beta_t3   = 1,
+               beta_t4   = 1,
+               end_t1    = 91,
+               end_t2    = 182,
+               end_t3    = 273,
+               end_t4    = 365,
+               epsilon   = 0)
+
+U <- structure(
+    c(0L, 0L, 0L, 0L, 0L, 0L, 2L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L,
+      0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L,
+      0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L,
+      1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L,
+      0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L,
+      0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L,
+      1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L,
+      0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L,
+      0L, 0L, 0L, 0L),
+    .Dim = c(12L, 11L))
+
+res <- run(model, threads = 1)
+stopifnot(identical(res@U, U))
+
+if (SimInf:::have_openmp()) {
+    res <- run(model, threads = 2)
+    stopifnot(identical(res@U, U))
+}
+
+## 2 Nodes
+## 3 Age categories
+## 2 Disease-states: Susceptible & Infected
+##
+## 6 individuals start in the susceptible compartments in node = 2,
+## with a zero probability of becoming infected.
+##
+## At t = 1, three individuals are moved to node = 1 using a select
+## matrix that can select from the susceptible S_1, S_2 and S_3.
+u0 <- structure(list(S_1 = c(0, 2),
+                     I_1 = c(0, 0),
+                     S_2 = c(0, 2),
+                     I_2 = c(0, 0),
+                     S_3 = c(0, 2),
+                     I_3 = c(0, 0)),
+                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
+                row.names = c(NA, -2L),
+                class = "data.frame")
+
+events <- structure(list(event      = 3,
+                         time       = 1,
+                         node       = 2,
+                         dest       = 1,
+                         n          = 3,
+                         proportion = 0,
+                         select     = 1,
+                         shift      = 0),
+                    .Names = c("event", "time", "node", "dest",
+                        "n", "proportion", "select", "shift"),
+                    row.names = c(NA, -1L), class = "data.frame")
+
+model <- SISe3(u0        = u0,
+               tspan     = 0:10,
+               events    = events,
+               phi       = rep(0, 2),
+               upsilon_1 = 0,
+               upsilon_2 = 0,
+               upsilon_3 = 0,
+               gamma_1   = 1,
+               gamma_2   = 1,
+               gamma_3   = 1,
+               alpha     = 0,
+               beta_t1   = 1,
+               beta_t2   = 1,
+               beta_t3   = 1,
+               beta_t4   = 1,
+               end_t1    = 91,
+               end_t2    = 182,
+               end_t3    = 273,
+               end_t4    = 365,
+               epsilon   = 0)
+
+## Add S_2 and S_3 to first column in select matrix
+model@events@E[3, 1] <- 1
+model@events@E[5, 1] <- 1
+
+S_expected <- structure(c(0L, 6L, 3L, 3L, 3L, 3L, 3L, 3L, 3L,
+                          3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L,
+                          3L, 3L, 3L, 3L),
+                        .Dim = c(2L, 11L))
+
+res <- run(model, threads = 1)
+stopifnot(identical(susceptible(res), S_expected))
+
+if (SimInf:::have_openmp()) {
+    res <- run(model, threads = 2)
+    stopifnot(identical(susceptible(res), S_expected))
+}
